@@ -18,6 +18,7 @@ def resolve_references(obj):
                     import pickle
                     import base64
                     import zlib
+                    from nerd_mega_compute.cloud.storage.data_size_utils import safe_unpickle
                     
                     # Parse bucket and key from S3 URI
                     parts = s3_uri[5:].split("/", 1)
@@ -38,7 +39,18 @@ def resolve_references(obj):
                         print(f"[REFERENCE_DEBUG] Successfully downloaded {data_size_mb:.2f} MB from S3")
                         print(f"[REFERENCE_DEBUG] First 100 bytes: {data[:100]}")
                         
-                        # Try multiple deserialization approaches
+                        # Try our safe unpickle first - this handles module path differences
+                        try:
+                            print("[REFERENCE_DEBUG] Trying safe_unpickle with compatibility handling")
+                            result = safe_unpickle(data)
+                            if result is not None:
+                                print(f"[REFERENCE_DEBUG] Successfully unpickled with safe_unpickle, result type: {type(result).__name__}")
+                                return result
+                            print("[REFERENCE_DEBUG] safe_unpickle returned None, trying other methods")
+                        except Exception as e:
+                            print(f"[REFERENCE_DEBUG] safe_unpickle failed: {str(e)}")
+                        
+                        # If safe_unpickle fails, try multiple deserialization approaches
                         try:
                             # Approach 1: If this is zlib-compressed pickled data
                             try:
@@ -47,15 +59,6 @@ def resolve_references(obj):
                                 print(f"[REFERENCE_DEBUG] Decompression successful, size: {len(decompressed) / (1024 * 1024):.2f} MB")
                                 result = pickle.loads(decompressed)
                                 print(f"[REFERENCE_DEBUG] Successfully unpickled data, result type: {type(result).__name__}")
-                                
-                                # Print detailed info about the result
-                                if hasattr(result, '__dict__'):
-                                    print(f"[REFERENCE_DEBUG] Result attributes: {dir(result)[:20]}")
-                                elif isinstance(result, dict):
-                                    print(f"[REFERENCE_DEBUG] Result keys: {list(result.keys())[:20]}")
-                                elif hasattr(result, 'shape'):
-                                    print(f"[REFERENCE_DEBUG] Result shape: {result.shape}")
-                                
                                 return result
                             except Exception as e:
                                 print(f"[REFERENCE_DEBUG] zlib+pickle approach failed: {str(e)}")
@@ -134,6 +137,21 @@ def resolve_references(obj):
                     import traceback
                     traceback.print_exc()
             return obj
+        # Check if it's a data reference dictionary - look for data_reference, s3Uri, and sizeMB fields
+        elif "data_reference" in obj and "s3Uri" in obj and "sizeMB" in obj:
+            print("[REFERENCE_DEBUG] Found direct data reference format")
+            data_id = obj["data_reference"]
+            s3_uri = obj["s3Uri"]
+            size_mb = obj["sizeMB"]
+            print(f"[REFERENCE_DEBUG] Resolving direct reference: {data_id}, S3 URI: {s3_uri}, Size: {size_mb}")
+            
+            # Create our standard format reference and recursively resolve
+            std_ref = {
+                "__nerd_data_reference": data_id,
+                "__nerd_s3_uri": s3_uri,
+                "__nerd_size_mb": size_mb
+            }
+            return resolve_references(std_ref)
         else:
             # Regular dictionary, process each value recursively
             return {k: resolve_references(v) for k, v in obj.items()}
