@@ -381,12 +381,51 @@ def fetch_nerd_data_reference(reference_obj):
     Returns:
         The fetched data, automatically deserialized based on its format
     """
+    print(f"[REF_DEBUG] fetch_nerd_data_reference called with: {type(reference_obj).__name__}")
+    
     if not isinstance(reference_obj, dict):
+        print(f"[REF_DEBUG] Not a dictionary reference object, returning unchanged: {type(reference_obj).__name__}")
         return reference_obj
 
+    # Handle our standard reference format
     if "__nerd_data_reference" in reference_obj:
         data_id = reference_obj["__nerd_data_reference"]
-        print(f"Fetching data reference: {data_id}")
-        return fetch_nerd_cloud_storage(data_id)
-
+        s3_uri = reference_obj.get("__nerd_s3_uri", "")
+        size_mb = reference_obj.get("__nerd_size_mb", "unknown")
+        
+        print(f"[REF_DEBUG] Fetching data reference: {data_id}, S3 URI: {s3_uri}, Size: {size_mb}")
+        
+        # Always use direct API fetch for consistency
+        api_key = get_api_key()
+        if not api_key:
+            raise ValueError("API_KEY is not set for cloud storage access")
+            
+        print(f"[REF_DEBUG] Using large_download.fetch_large_file for consistency")
+        from .large_download import fetch_large_file
+        
+        # Use the large file download function directly
+        result = fetch_large_file(data_id, api_key)
+        print(f"[REF_DEBUG] Fetch completed, result type: {type(result).__name__}")
+        
+        return result
+    
+    # Handle the bytes_reference format from serialize_with_chunking
+    elif "type" in reference_obj and reference_obj["type"] == "bytes_reference" and "value" in reference_obj:
+        ref_data = reference_obj["value"]
+        if isinstance(ref_data, dict) and "data_reference" in ref_data:
+            data_id = ref_data["data_reference"]
+            s3_uri = ref_data.get("s3Uri", "")
+            size_mb = ref_data.get("sizeMB", "unknown")
+            
+            print(f"[REF_DEBUG] Resolving bytes_reference: {data_id}, S3 URI: {s3_uri}, Size: {size_mb}")
+            
+            # Convert to our standard format and recursively handle
+            std_ref = {
+                "__nerd_data_reference": data_id,
+                "__nerd_s3_uri": s3_uri,
+                "__nerd_size_mb": size_mb
+            }
+            return fetch_nerd_data_reference(std_ref)
+    
+    print(f"[REF_DEBUG] Not a recognized data reference format, returning unchanged")
     return reference_obj
